@@ -57,7 +57,6 @@
 #ifdef USE_DCC64
 #define BIG_STR_TO_INT(x) strtoull(x,NULL,10)
 #else
-#warning Not able to use 64-bit DCC
 #define BIG_STR_TO_INT(x) strtoul(x,NULL,10)
 #endif
 
@@ -73,7 +72,7 @@ struct dccstat_info dccstat[] = {
 };
 
 static int dcc_global_throttle;	/* 0x1 = sends, 0x2 = gets */
-static int dcc_sendcpssum, dcc_getcpssum;
+/*static*/ int dcc_sendcpssum, dcc_getcpssum;
 
 static struct DCC *new_dcc (void);
 static void dcc_close (struct DCC *dcc, int dccstat, int destroy);
@@ -87,9 +86,8 @@ static int new_id()
 	if (id == 0)
 	{
 		/* start the first ID at a random number for pseudo security */
-		srand (time (0));
 		/* 1 - 255 */
-		id = 1 + (int) ((float)255 * rand()/(RAND_MAX+1.0));
+		id = RAND_INT(255) + 1;
 		/* ignore overflows, since it can go to 2 billion */
 	}
 	return id++;
@@ -328,8 +326,8 @@ dcc_close (struct DCC *dcc, int dccstat, int destroy)
 			{			
 				/* mgl: change this to use destfile_fs for correctness and to */
 				/* handle the case where dccwithnick is set */
-				download_move_to_completed_dir(prefs.dccdir, prefs.dcc_completed_dir, 
-					dcc->destfile_fs, prefs.dccpermissions);
+				move_file_utf8 (prefs.dccdir, prefs.dcc_completed_dir, 
+									 file_part (dcc->destfile), prefs.dccpermissions);
 			}
 
 		}
@@ -457,7 +455,7 @@ dcc_write_chat (char *nick, char *text)
 				1 - the dcc is closed! */
 
 static int
-dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
+dcc_chat_line (struct DCC *dcc, char *line)
 {
 	session *sess;
 	char *word[PDIWORDS];
@@ -467,6 +465,7 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 	int ret, i;
 	int len;
 	gsize utf_len;
+	char portbuf[32];
 
 	len = strlen (line);
 	if (dcc->serv->using_cp1255)
@@ -493,11 +492,11 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 	if (!sess)
 		sess = dcc->serv->front_session;
 
-	sprintf (tbuf, "%d", dcc->port);
+	sprintf (portbuf, "%d", dcc->port);
 
 	word[0] = "DCC Chat Text";
 	word[1] = net_ip (dcc->addr);
-	word[2] = tbuf;
+	word[2] = portbuf;
 	word[3] = dcc->nick;
 	word[4] = line;
 	for (i = 5; i < PDIWORDS; i++)
@@ -532,7 +531,7 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		po = strchr (line + 8, '\001');
 		if (po)
 			po[0] = 0;
-		inbound_action (sess, dcc->serv->nick, dcc->nick, line + 8, FALSE);
+		inbound_action (sess, dcc->serv->nick, dcc->nick, line + 8, FALSE, FALSE);
 	} else
 	{
 		inbound_privmsg (dcc->serv, dcc->nick, "", line, FALSE);
@@ -548,9 +547,8 @@ static gboolean
 dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	int i, len, dead;
-	char tbuf[1226];
-	char lbuf[1026];
-	char *temp;
+	char portbuf[32];
+	char lbuf[2050];
 
 	while (1)
 	{
@@ -572,9 +570,9 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				if (would_block ())
 					return TRUE;
 			}
-			sprintf (tbuf, "%d", dcc->port);
+			sprintf (portbuf, "%d", dcc->port);
 			EMIT_SIGNAL (XP_TE_DCCCHATF, dcc->serv->front_session, dcc->nick,
-							 net_ip (dcc->addr), tbuf,
+							 net_ip (dcc->addr), portbuf,
 							 errorstring ((len < 0) ? sock_error () : 0), 0);
 			dcc_close (dcc, STAT_FAILED, FALSE);
 			return TRUE;
@@ -589,16 +587,7 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				break;
 			case '\n':
 				dcc->dccchat->linebuf[dcc->dccchat->pos] = 0;
-
-				if (prefs.stripcolor)
-				{
-					temp = strip_color (dcc->dccchat->linebuf, -1, 1, 1);
-					dead = dcc_chat_line (dcc, temp, tbuf);
-					free (temp);
-				} else
-				{
-					dead = dcc_chat_line (dcc, dcc->dccchat->linebuf, tbuf);
-				}
+				dead = dcc_chat_line (dcc, dcc->dccchat->linebuf);
 
 				if (dead || !dcc->dccchat) /* the dcc has been closed, don't use (DCC *)! */
 					return TRUE;
@@ -609,7 +598,7 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				break;
 			default:
 				dcc->dccchat->linebuf[dcc->dccchat->pos] = lbuf[i];
-				if (dcc->dccchat->pos < 1022)
+				if (dcc->dccchat->pos < (sizeof (dcc->dccchat->linebuf) - 1))
 					dcc->dccchat->pos++;
 			}
 			i++;
@@ -747,9 +736,9 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		if (dcc->pos >= dcc->size)
 		{
 			dcc_send_ack (dcc);
-			dcc_calc_average_cps (dcc);
-			sprintf (buf, "%d", dcc->cps);
 			dcc_close (dcc, STAT_DONE, FALSE);
+			dcc_calc_average_cps (dcc);	/* this must be done _after_ dcc_close, or dcc_remove_from_sum will see the wrong value in dcc->cps */
+			sprintf (buf, "%d", dcc->cps);
 			EMIT_SIGNAL (XP_TE_DCCRECVCOMP, dcc->serv->front_session,
 							 dcc->file, dcc->destfile, dcc->nick, buf, 0);
 			return TRUE;
@@ -887,11 +876,11 @@ dcc_connect (struct DCC *dcc)
 		/* possible problems with filenames containing spaces? */
 		if (dcc->type == TYPE_RECV)
 			snprintf (tbuf, sizeof (tbuf), strchr (dcc->file, ' ') ?
-					"DCC SEND \"%s\" %lu %d %"DCC_SFMT" %d" :
-					"DCC SEND %s %lu %d %"DCC_SFMT" %d", dcc->file,
+					"DCC SEND \"%s\" %u %d %"DCC_SFMT" %d" :
+					"DCC SEND %s %u %d %"DCC_SFMT" %d", dcc->file,
 					dcc->addr, dcc->port, dcc->size, dcc->pasvid);
 		else
-			snprintf (tbuf, sizeof (tbuf), "DCC CHAT chat %lu %d %d",
+			snprintf (tbuf, sizeof (tbuf), "DCC CHAT chat %u %d %d",
 				dcc->addr, dcc->port, dcc->pasvid);
 		dcc->serv->p_ctcp (dcc->serv, dcc->nick, tbuf);
 	}
@@ -1020,8 +1009,8 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	if (dcc->pos >= dcc->size && dcc->ack >= (dcc->size & 0xffffffff))
 	{
 		dcc->ack = dcc->size;	/* force 100% ack for >4 GB */
-		dcc_calc_average_cps (dcc);
 		dcc_close (dcc, STAT_DONE, FALSE);
+		dcc_calc_average_cps (dcc);	/* this must be done _after_ dcc_close, or dcc_remove_from_sum will see the wrong value in dcc->cps */
 		sprintf (buf, "%d", dcc->cps);
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
@@ -1099,12 +1088,35 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	return TRUE;
 }
 
+guint32
+dcc_get_my_address (void)	/* the address we'll tell the other person */
+{
+	struct hostent *dns_query;
+	guint32 addr = 0;
+
+	if (prefs.ip_from_server && prefs.dcc_ip)
+		addr = prefs.dcc_ip;
+	else if (prefs.dcc_ip_str[0])
+	{
+	   dns_query = gethostbyname ((const char *) prefs.dcc_ip_str);
+
+	   if (dns_query != NULL &&
+	       dns_query->h_length == 4 &&
+	       dns_query->h_addr_list[0] != NULL)
+		{
+			/*we're offered at least one IPv4 address: we take the first*/
+			addr = *((guint32*) dns_query->h_addr_list[0]);
+		}
+	}
+
+	return addr;
+}
+
 static int
 dcc_listen_init (struct DCC *dcc, session *sess)
 {
-	unsigned long my_addr;
+	guint32 my_addr;
 	struct sockaddr_in SAddr;
-	struct hostent *dns_query;
 	int i, bindretval = -1;
 	socklen_t len;
 
@@ -1169,24 +1181,7 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 	/*if we have a dcc_ip, we use that, so the remote client can connect*/
 	/*else we try to take an address from dcc_ip_str*/
 	/*if something goes wrong we tell the client to connect to our LAN ip*/
-
-	dcc->addr = 0;
-
-	if (prefs.ip_from_server != 0 && prefs.dcc_ip != 0)
-		dcc->addr = prefs.dcc_ip;
-	else if (prefs.dcc_ip_str[0])
-	{
-	   dns_query = gethostbyname ((const char *) prefs.dcc_ip_str);
-
-	   if (dns_query != NULL &&
-	       dns_query->h_length == 4 &&
-	       dns_query->h_addr_list[0] != NULL)
-		  {
-		    /*we're offered at least one IPv4 address: we take the first*/
-
-		    dcc->addr = (unsigned long) *((guint32*) dns_query->h_addr_list[0]);
-		  }
-	}
+	dcc->addr = dcc_get_my_address ();
 
 	/*if nothing else worked we use the address we bound to*/
 	if (dcc->addr == 0)
@@ -1324,8 +1319,8 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 						} else
 						{
 							snprintf (outbuf, sizeof (outbuf), (havespaces) ?
-									"DCC SEND \"%s\" %lu %d %"DCC_SFMT :
-									"DCC SEND %s %lu %d %"DCC_SFMT,
+									"DCC SEND \"%s\" %u %d %"DCC_SFMT :
+									"DCC SEND %s %u %d %"DCC_SFMT,
 									file_part (dcc->file), dcc->addr,
 									dcc->port, dcc->size);
 						}
@@ -1555,7 +1550,7 @@ dcc_chat (struct session *sess, char *nick, int passive)
 						 dcc->port, dcc->pasvid);
 		} else
 		{
-			snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat %lu %d",
+			snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat %u %d",
 						 dcc->addr, dcc->port);
 		}
 		dcc->serv->p_ctcp (dcc->serv, nick, outbuf);
@@ -1572,6 +1567,36 @@ dcc_malformed (struct session *sess, char *nick, char *data)
 	EMIT_SIGNAL (XP_TE_MALFORMED, sess, nick, data, NULL, NULL, 0);
 }
 
+/* is the destination file the same? new_dcc is not opened yet */
+
+static int
+is_same_file (struct DCC *dcc, struct DCC *new_dcc)
+{
+	struct stat st_a, st_b;
+
+	/* if it's the same filename, must be same */
+	if (strcmp (dcc->destfile, new_dcc->destfile) == 0)
+		return TRUE;
+
+	/* now handle case-insensitive Filesystems: HFS+, FAT */
+#ifdef WIN32
+#error implement me!
+#else
+	/* this fstat() shouldn't really fail */
+	if ((dcc->fp == -1 ? stat (dcc->destfile_fs, &st_a) : fstat (dcc->fp, &st_a)) == -1)
+		return FALSE;
+	if (stat (new_dcc->destfile_fs, &st_b) == -1)
+		return FALSE;
+
+	/* same inode, same device, same file! */
+	if (st_a.st_ino == st_b.st_ino &&
+		 st_a.st_dev == st_b.st_dev)
+		return TRUE;
+#endif
+
+	return FALSE;
+}
+
 static int
 is_resumable (struct DCC *dcc)
 {
@@ -1614,7 +1639,7 @@ is_resumable (struct DCC *dcc)
 			if (d->type == TYPE_RECV && d->dccstat != STAT_ABORTED &&
 				 d->dccstat != STAT_DONE && d->dccstat != STAT_FAILED)
 			{
-				if (d != dcc && strcmp (d->destfile, dcc->destfile) == 0)
+				if (d != dcc && is_same_file (d, dcc))
 				{
 					dcc->resume_error = 3;	/* dccgui.c uses it */
 					dcc->resumable = 0;
@@ -1681,7 +1706,7 @@ dcc_deny_chat (void *ud)
 }
 
 static struct DCC *
-dcc_add_chat (session *sess, char *nick, int port, unsigned long addr, int pasvid)
+dcc_add_chat (session *sess, char *nick, int port, guint32 addr, int pasvid)
 {
 	struct DCC *dcc;
 
@@ -1721,7 +1746,7 @@ dcc_add_chat (session *sess, char *nick, int port, unsigned long addr, int pasvi
 }
 
 static struct DCC *
-dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, unsigned long addr, int pasvid)
+dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, guint32 addr, int pasvid)
 {
 	struct DCC *dcc;
 	char tbuf[512];
@@ -1792,7 +1817,7 @@ dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, un
 			fe_dcc_add (dcc);
 	}
 	sprintf (tbuf, "%"DCC_SFMT, size);
-	snprintf (tbuf + 24, 300, "%s:%d", net_ip (dcc->addr), dcc->port);
+	snprintf (tbuf + 24, 300, "%s:%d", net_ip (addr), port);
 	EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
 					 file, tbuf, tbuf + 24, 0);
 
@@ -1807,7 +1832,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 	struct DCC *dcc;
 	char *type = word[5];
 	int port, pasvid = 0;
-	unsigned long addr;
+	guint32 addr;
 	DCC_SIZE size;
 	int psend = 0;
 
