@@ -66,13 +66,10 @@ void
 notify_save (void)
 {
 	int fh;
-	char buf[256];
 	struct notify *notify;
 	GSList *list = notify_list;
 
-	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir_fs ());
-
-	fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
+	fh = xchat_open_file ("notify.conf", O_TRUNC | O_WRONLY | O_CREAT, 0600, XOF_DOMODE);
 	if (fh != -1)
 	{
 		while (list)
@@ -92,12 +89,10 @@ notify_load (void)
 	int fh;
 	char buf[256];
 
-	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir_fs ());
-
-	fh = open (buf, O_RDONLY | OFLAGS);
+	fh = xchat_open_file ("notify.conf", O_RDONLY, 0, 0);
 	if (fh != -1)
 	{
-		while (waitline (fh, buf, sizeof buf) != -1)
+		while (waitline (fh, buf, sizeof buf, FALSE) != -1)
 		{
 			if (buf[0] != '#' && buf[0] != 0)
 				notify_adduser (buf);
@@ -397,9 +392,9 @@ notify_showlist (struct session *sess)
 		notify = (struct notify *) list->data;
 		servnot = notify_find_server_entry (notify, sess->server);
 		if (servnot && servnot->ison)
-			sprintf (outbuf, _("  %-20s online\n"), notify->name);
+			snprintf (outbuf, sizeof (outbuf), _("  %-20s online\n"), notify->name);
 		else
-			sprintf (outbuf, _("  %-20s offline\n"), notify->name);
+			snprintf (outbuf, sizeof (outbuf), _("  %-20s offline\n"), notify->name);
 		PrintText (sess, outbuf);
 		list = list->next;
 	}
@@ -451,7 +446,14 @@ notify_adduser (char *name)
 	if (notify)
 	{
 		memset (notify, 0, sizeof (struct notify));
-		notify->name = strdup (name);
+		if (strlen (name) >= NICKLEN)
+		{
+			notify->name = malloc (NICKLEN);
+			safe_strcpy (notify->name, name, NICKLEN);
+		} else
+		{
+			notify->name = strdup (name);
+		}
 		notify->server_list = 0;
 		notify_list = g_slist_prepend (notify_list, notify);
 		notify_checklist ();
