@@ -24,6 +24,7 @@
 #include <unistd.h>
 
 #include "xchat.h"
+#include <glib/ghash.h>
 
 #include "cfgfiles.h"
 #include "fe.h"
@@ -47,8 +48,6 @@ static const struct defaultserver def[] =
 {
 	{"2600net",	0},
 	{0,			"irc.2600.net"},
-	{0,			"ptolomea.2600.net"},
-	{0,			"collective.2600.net"},
 
 	{"AccessIRC",	0},
 	{0,			"irc.accessirc.net"},
@@ -68,23 +67,23 @@ static const struct defaultserver def[] =
 #endif
 #ifdef USE_OPENSSL
 	{0,			"irc.data.lt/+6668"},
+	{0,			"irc-ssl.omnitel.net/+6668"},
+	{0,			"irc-ssl.le.lt/+9999"},
 #endif
 	{0,			"irc.data.lt"},
+	{0,			"irc.omnitel.net"},
 	{0,			"irc.ktu.lt"},
 	{0,			"irc.le.lt"},
-	{0,			"irc.omnitel.net"},
 	{0,			"irc.takas.lt"},
 	{0,			"irc.5ci.net"},
-	{0,			"irc.interneka.lt"},
-	{0,			"irc.elekta.lt"},
-	{0,			"irc.if.lt"},
+	{0,			"irc.kis.lt"},
 
 	{"AmigaNet",	0},
 	{0,			"irc.amiganet.org"},
 	{0,			"us.amiganet.org"},
 	{0,			"uk.amiganet.org"},
-	{0,			"no.amiganet.org"},
-	{0,			"au.amiganet.org"},
+/*	{0,			"no.amiganet.org"},
+	{0,			"au.amiganet.org"},*/
 
 	{"ARCNet",	0},
 	{0,			"se1.arcnet.vapor.com"},
@@ -99,7 +98,7 @@ static const struct defaultserver def[] =
 	{0,			"nl3.arcnet.vapor.com"},
 	{0,			"uk1.arcnet.vapor.com"},
 	{0,			"uk2.arcnet.vapor.com"},
-	{0,			"uk3.arcnet.vapor.com"},
+/*	{0,			"uk3.arcnet.vapor.com"},*/
 	{0,			"fr1.arcnet.vapor.com"},
 
 	{"AstroLink",	0},
@@ -110,18 +109,18 @@ static const struct defaultserver def[] =
 	{0,			"us.austnet.org"},
 	{0,			"ca.austnet.org"},
 
-	{"AxeNet",	0},
+/*	{"AxeNet",	0},
 	{0,			"irc.axenet.org"},
 	{0,			"angel.axenet.org"},
 	{0,			"energy.axenet.org"},
-	{0,			"python.axenet.org"},
+	{0,			"python.axenet.org"},*/
 
 	{"AzzurraNet",	0},
 	{0,			"irc.azzurra.org"},
 	{0,			"crypto.azzurra.org"},
 
-	{"Buzzpot", 0},
-	{0,			"irc.chatspike.net"},
+	{"Beirut", 0},
+	{0,			"irc.beirut.com"},
 
 	{"ChatJunkies",	0, "#xchat"},
 	{0,			"irc.chatjunkies.org"},
@@ -135,10 +134,13 @@ static const struct defaultserver def[] =
 	{0,			"us.chatsociety.net"},
 	{0,			"eu.chatsociety.net"},
 
+	{"ChatSpike", 0},
+	{0,			"irc.chatspike.net"},
+
 	{"CoolChat",	0},
 	{0,			"irc.coolchat.net"},
-	{0,			"unix.coolchat.net"},
-	{0,			"toronto.coolchat.net"},
+/*	{0,			"unix.coolchat.net"},
+	{0,			"toronto.coolchat.net"},*/
 
 	{"Criten", 0},
 	{0,			"irc.criten.net"},
@@ -162,7 +164,7 @@ static const struct defaultserver def[] =
 	{0,			"irc.Qeast.net"},
 	{0,			"irc.efnet.pl"},
 	{0,			"efnet.demon.co.uk"},
-	{0,			"irc.lagged.org"},
+/*	{0,			"irc.lagged.org"},*/
 	{0,			"irc.lightning.net"},
 	{0,			"irc.mindspring.com"},
 	{0,			"irc.easynews.com"},
@@ -194,38 +196,37 @@ static const struct defaultserver def[] =
 
 	{"FEFNet",	0},
 	{0,			"irc.fef.net"},
-	{0,			"irc.villagenet.com"},
 	{0,			"irc.ggn.net"},
 	{0,			"irc.vendetta.com"},
 
 	{"FreeNode",	0},
 	{0,				"irc.freenode.net"},
 
-	{"Freeworld",	0},
+/*	{"Freeworld",	0},
 	{0,			"kabel.freeworld.nu"},
-	{0,			"irc.freeworld.nu"},
+	{0,			"irc.freeworld.nu"},*/
 
 	{"GalaxyNet",	0},
 	{0,			"irc.galaxynet.org"},
-	{0,			"sprynet.us.galaxynet.org"},
-	{0,			"atlanta.ga.us.galaxynet.org"},
+/*	{0,			"sprynet.us.galaxynet.org"},
+	{0,			"atlanta.ga.us.galaxynet.org"},*/
 
 	{"GamesNET",	0},
 	{0,				"irc.gamesnet.net"},
-	{0,				"irc.us.gamesnet.net"},
+/*	{0,				"irc.us.gamesnet.net"},
 	{0,				"east.us.gamesnet.net"},
-	{0,				"west.us.gamesnet.net"},
+	{0,				"west.us.gamesnet.net"},*/
 	{0,				"irc.ca.gamesnet.net"},
 	{0,				"irc.eu.gamesnet.net"},
 
 	{"German-Elite",	0},
 	{0,			"dominion.german-elite.net"},
 	{0,			"komatu.german-elite.net"},
-	{0,			"liberty.german-elite.net"},
+/*	{0,			"liberty.german-elite.net"},*/
 
 	{"GimpNet",		0},
 	{0,			"irc.gimp.org"},
-	{0,			"irc.au.gimp.org"},
+/*	{0,			"irc.au.gimp.org"},*/
 	{0,			"irc.us.gimp.org"},
 
 	{"HabberNet",	0},
@@ -234,11 +235,14 @@ static const struct defaultserver def[] =
 	{"Hashmark",	0},
 	{0,			"irc.hashmark.net"},
 
-	{"Infinity-IRC",	0},
+	{"IdleMonkeys", 0},
+	{0,			"irc.idlemonkeys.net"},
+
+/*	{"Infinity-IRC",	0},
 	{0,			"Atlanta.GA.US.Infinity-IRC.Org"},
 	{0,			"Babylon.NY.US.Infinity-IRC.Org"},
 	{0,			"Sunshine.Ca.US.Infinity-IRC.Org"},
-	{0,			"IRC.Infinity-IRC.Org"},
+	{0,			"IRC.Infinity-IRC.Org"},*/
 
 	{"insiderZ.DE",	0},
 	{0,			"irc.insiderz.de/6667"},
@@ -255,13 +259,13 @@ static const struct defaultserver def[] =
 	{0,				"irc.ircnet.com"},
 	{0,				"irc.stealth.net/6668"},
 	{0,				"ircnet.demon.co.uk"},
-	{0,				"ircnet.hinet.hr"},
+/*	{0,				"ircnet.hinet.hr"},*/
 	{0,				"irc.datacomm.ch"},
-	{0,				"ircnet.kaptech.fr"},
-	{0,				"ircnet.easynet.co.uk"},
+/*	{0,				"ircnet.kaptech.fr"},
+	{0,				"ircnet.easynet.co.uk"},*/
 	{0,				"random.ircd.de"},
 	{0,				"ircnet.netvision.net.il"},
-	{0,				"irc.seed.net.tw"},
+/*	{0,				"irc.seed.net.tw"},*/
 	{0,				"irc.cs.hut.fi"},
 
 	{"Irctoo.net",	0},
@@ -278,10 +282,10 @@ static const struct defaultserver def[] =
 	{"LinkNet",	0},
 	{0,			"irc.link-net.org"},
 	{0,			"irc.no.link-net.org"},
-	{0,			"irc.gamesden.net.au"},
+/*	{0,			"irc.gamesden.net.au"},*/
 	{0,			"irc.bahnhof.se"},
-	{0,			"irc.kinexuseurope.co.uk"},
-	{0,			"irc.gamiix.com"},
+/*	{0,			"irc.kinexuseurope.co.uk"},
+	{0,			"irc.gamiix.com"},*/
 
 	{"MagicStar",	0},
 	{0,			"irc.magicstar.net"},
@@ -289,6 +293,12 @@ static const struct defaultserver def[] =
 	{"Majistic",	0},
 	{0,			"irc.majistic.net"},
 
+	{"MindForge",	0},
+	{0,			"irc.mindforge.org"},
+
+	{"MIXXnet",		0},
+	{0,			"irc.mixxnet.net"},
+
 	{"NeverNET",	0},
 	{0,			"irc.nevernet.net"},
 	{0,			"imagine.nevernet.net"},
@@ -352,7 +362,7 @@ static const struct defaultserver def[] =
 	{0,			"uc.PTnet.org"},
 	{0,			"ualg.ptnet.org"},
 	{0,			"madinfo.PTnet.org"},
-	{0,			"isep.PTnet.org"},
+/*	{0,			"isep.PTnet.org"},*/
 	{0,			"ua.PTnet.org"},
 	{0,			"ipg.PTnet.org"},
 	{0,			"isec.PTnet.org"},
@@ -374,7 +384,7 @@ static const struct defaultserver def[] =
 	{"RebelChat",	0},
 	{0,			"irc.rebelchat.org"},
 
-	{"Recycled-IRC",  0},
+/*	{"Recycled-IRC",  0},
 	{0,			"irc.recycled-irc.org"},
 	{0,			"vermin.recycled-irc.org"},
 	{0,			"waste.recycled-irc.org"},
@@ -382,7 +392,7 @@ static const struct defaultserver def[] =
 	{0,			"trash.recycled-irc.org"},
 	{0,			"unwashed.recycled-irc.org"},
 	{0,			"garbage.recycled-irc.org"},
-	{0,			"dustbin.recycled-irc.org"},
+	{0,			"dustbin.recycled-irc.org"},*/
 
 	{"RizeNET", 0},
 	{0,			"irc.rizenet.org"},
@@ -391,7 +401,7 @@ static const struct defaultserver def[] =
 	{0,			"lisa.rizenet.org"},
 	{0,			"scott.rizenet.org"},
 
-	{"RusNet", 0, 0, "KOI8-R"},
+	{"RusNet", 0, 0, "KOI8-R (Cyrillic)"},
 	{0,			"irc.tomsk.net"},
 	{0,			"irc.rinet.ru"},
 	{0,			"irc.run.net"},
@@ -434,9 +444,6 @@ static const struct defaultserver def[] =
 	{"TNI3",			0},
 	{0,			"irc.tni3.com"},
 
-	{"TopGamers",	0},
-	{0,			"irc.topgamers.net"},
-
 	{"UnderNet",	0},
 	{0,			"us.undernet.org"},
 	{0,			"eu.undernet.org"},
@@ -449,15 +456,18 @@ static const struct defaultserver def[] =
 	{0,			"irc.telecoms.bg"},
 	{0,			"irc.tu-varna.edu"},
 
+	{"Whiffle",	0},
+	{0,			"irc.whiffle.org"},
+
 	{"Worldnet",		0},
 	{0,			"irc.worldnet.net"},
 	{0,			"irc.fr.worldnet.net"},
 
 	{"Xentonix.net",	0},
 	{0,			"irc.ffm.de.eu.xentonix.net"},
-	{0,			"irc.brs.de.eu.xentonix.net"},
+/*	{0,			"irc.brs.de.eu.xentonix.net"},
 	{0,			"irc.stg.ch.eu.xentonix.net"},
-	{0,			"irc.hou.tx.us.xentonix.net"},
+	{0,			"irc.hou.tx.us.xentonix.net"},*/
 	{0,			"irc.kar.de.eu.xentonix.net"},
 	{0,			"irc.vie.at.eu.xentonix.net"},
 
@@ -475,7 +485,7 @@ GSList *network_list = 0;
 
 
 void
-servlist_connect (session *sess, ircnet *net)
+servlist_connect (session *sess, ircnet *net, gboolean join)
 {
 	ircserver *ircserv;
 	GSList *list;
@@ -483,7 +493,7 @@ servlist_connect (session *sess, ircnet *net)
 	server *serv;
 
 	if (!sess)
-		sess = new_ircwindow (NULL, NULL, SESS_SERVER, 0);
+		sess = new_ircwindow (NULL, NULL, SESS_SERVER, TRUE);
 
 	serv = sess->server;
 
@@ -496,12 +506,16 @@ servlist_connect (session *sess, ircnet *net)
 	/* incase a protocol switch is added to the servlist gui */
 	server_fill_her_up (sess->server);
 
-	sess->willjoinchannel[0] = 0;
-	serv->password[0] = 0;
+	if (join)
+	{
+		sess->willjoinchannel[0] = 0;
 
-	if (net->autojoin)
-		safe_strcpy (sess->willjoinchannel, net->autojoin,
-						 sizeof (sess->willjoinchannel));
+		if (net->autojoin)
+			safe_strcpy (sess->willjoinchannel, net->autojoin,
+							 sizeof (sess->willjoinchannel));
+	}
+
+	serv->password[0] = 0;
 	if (net->pass)
 		safe_strcpy (serv->password, net->pass, sizeof (serv->password));
 
@@ -549,7 +563,7 @@ servlist_connect (session *sess, ircnet *net)
 }
 
 int
-servlist_connect_by_netname (session *sess, char *network)
+servlist_connect_by_netname (session *sess, char *network, gboolean join)
 {
 	ircnet *net;
 	GSList *list = network_list;
@@ -560,7 +574,7 @@ servlist_connect_by_netname (session *sess, char *network)
 
 		if (strcasecmp (net->name, network) == 0)
 		{
-			servlist_connect (sess, net);
+			servlist_connect (sess, net, join);
 			return 1;
 		}
 
@@ -603,7 +617,7 @@ servlist_auto_connect (session *sess)
 
 		if (net->flags & FLAG_AUTO_CONNECT)
 		{
-			servlist_connect (sess, net);
+			servlist_connect (sess, net, TRUE);
 			ret = 1;
 		}
 
@@ -618,7 +632,7 @@ servlist_cycle_cb (server *serv)
 {
 	PrintTextf (serv->server_session,
 		_("Cycling to next server in %s...\n"), ((ircnet *)serv->network)->name);
-	servlist_connect (serv->server_session, serv->network);
+	servlist_connect (serv->server_session, serv->network, TRUE);
 
 	return 0;
 }
@@ -650,7 +664,7 @@ servlist_cycle (server *serv)
 			if (del)
 				serv->recondelay_tag = fe_timeout_add (del, servlist_cycle_cb, serv);
 			else
-				servlist_connect (serv->server_session, net);
+				servlist_connect (serv->server_session, net, TRUE);
 
 			return TRUE;
 		}
@@ -682,8 +696,38 @@ servlist_server_find (ircnet *net, char *name, int *pos)
 	return NULL;
 }
 
+/* find a network (e.g. (ircnet *) to "FreeNode") from a hostname
+   (e.g. "irc.eu.freenode.net") */
+
 ircnet *
-servlist_net_find (char *name, int *pos)
+servlist_net_find_from_server (char *server_name)
+{
+	GSList *list = network_list;
+	GSList *slist;
+	ircnet *net;
+	ircserver *serv;
+
+	while (list)
+	{
+		net = list->data;
+
+		slist = net->servlist;
+		while (slist)
+		{
+			serv = slist->data;
+			if (strcasecmp (serv->hostname, server_name) == 0)
+				return net;
+			slist = slist->next;
+		}
+
+		list = list->next;
+	}
+
+	return NULL;
+}
+
+ircnet *
+servlist_net_find (char *name, int *pos, int (*cmpfunc) (const char *, const char *))
 {
 	GSList *list = network_list;
 	ircnet *net;
@@ -692,7 +736,7 @@ servlist_net_find (char *name, int *pos)
 	while (list)
 	{
 		net = list->data;
-		if (strcmp (net->name, name) == 0)
+		if (cmpfunc (net->name, name) == 0)
 		{
 			if (pos)
 				*pos = i;
@@ -750,6 +794,8 @@ servlist_net_remove (ircnet *net)
 
 	if (net->nick)
 		free (net->nick);
+	if (net->nick2)
+		free (net->nick2);
 	if (net->user)
 		free (net->user);
 	if (net->real)
@@ -819,10 +865,12 @@ servlist_load_defaults (void)
 				net->autojoin = strdup (def[i].channel);
 			if (def[i].charset)
 			{
+#ifdef WIN32
 				free (net->encoding);
+#endif
 				net->encoding = strdup (def[i].charset);
 			}
-			if (!strcmp (def[i].network, "ChatJunkies"))
+			if (g_str_hash (def[i].network) == 0x8e1b96f7)
 				prefs.slist_select = j;
 			j++;
 		} else
@@ -844,8 +892,7 @@ servlist_load (void)
 	char *tmp;
 	ircnet *net = NULL;
 
-	snprintf (buf, sizeof (buf), "%s/servlist_.conf", get_xdir_fs ());
-	fp = fopen (buf, "r");
+	fp = xchat_fopen_file ("servlist_.conf", "r", 0);
 	if (!fp)
 		return FALSE;
 
@@ -861,6 +908,9 @@ servlist_load (void)
 			case 'I':
 				net->nick = strdup (buf + 2);
 				break;
+			case 'i':
+				net->nick2 = strdup (buf + 2);
+				break;
 			case 'U':
 				net->user = strdup (buf + 2);
 				break;
@@ -960,12 +1010,15 @@ servlist_save (void)
 	ircserver *serv;
 	GSList *list;
 	GSList *hlist;
+#ifndef WIN32
 	int first = FALSE;
 
 	snprintf (buf, sizeof (buf), "%s/servlist_.conf", get_xdir_fs ());
 	if (access (buf, F_OK) != 0)
 		first = TRUE;
-	fp = fopen (buf, "w");
+#endif
+
+	fp = xchat_fopen_file ("servlist_.conf", "w", 0);
 	if (!fp)
 		return FALSE;
 
@@ -983,6 +1036,8 @@ servlist_save (void)
 		fprintf (fp, "N=%s\n", net->name);
 		if (net->nick)
 			fprintf (fp, "I=%s\n", net->nick);
+		if (net->nick2)
+			fprintf (fp, "i=%s\n", net->nick2);
 		if (net->user)
 			fprintf (fp, "U=%s\n", net->user);
 		if (net->real)
@@ -1001,7 +1056,7 @@ servlist_save (void)
 			{
 				snprintf (buf, sizeof (buf), _("Warning: \"%s\" character set is unknown. No conversion will be applied for network %s."),
 							 net->encoding, net->name);
-				fe_message (buf, FALSE);
+				fe_message (buf, FE_MSG_WARN);
 			}
 		}
 
