@@ -68,7 +68,7 @@ menu_about (GtkWidget * wid, gpointer sess)
 #endif
 
 	gtk_widget_show (gnome_about_new ("X-Chat", VERSION,
-							"(C) 1998-2003 Peter Zelezny", author, buf, 0));
+							"(C) 1998-2005 Peter Zelezny", author, buf, 0));
 }
 
 #else
@@ -99,7 +99,7 @@ menu_about (GtkWidget * wid, gpointer sess)
 	about = gtk_dialog_new ();
 	gtk_window_set_position (GTK_WINDOW (about), GTK_WIN_POS_CENTER);
 	gtk_window_set_resizable (GTK_WINDOW (about), FALSE);
-	gtk_window_set_title (GTK_WINDOW (about), _("About X-Chat"));
+	gtk_window_set_title (GTK_WINDOW (about), _("About XChat"));
 	if (parent_window)
 		gtk_window_set_transient_for (GTK_WINDOW (about), parent_window);
 	g_signal_connect (G_OBJECT (about), "destroy",
@@ -115,18 +115,30 @@ menu_about (GtkWidget * wid, gpointer sess)
 	gtk_container_add (GTK_CONTAINER (vbox), label);
 	g_get_charset (&locale);
 	(snprintf) (buf, sizeof (buf),
-				"<span size=\"x-large\"><b>X-Chat "VERSION"</b></span>\n\n"
-				"A multiplatform IRC Client\n\n"
+				"<span size=\"x-large\"><b>XChat "VERSION"</b></span>\n\n"
+				"%s\n\n"
 				"%s\n"
-				"<b>Charset</b>: %s <b>Renderer</b>: %s\n"
+				"<b>Charset</b>: %s "
+#ifdef WIN32 
+				"<b>GTK+</b>: %i.%i.%i\n"
+#else
+				"<b>Renderer</b>: %s\n"
+#endif
 				"<b>Compiled</b>: "__DATE__"\n\n"
-				"\302\251 1998-2003 Peter \305\275elezn\303\275 &lt;zed@xchat.org>",
+				"<small>\302\251 1998-2006 Peter \305\275elezn\303\275 &lt;zed@xchat.org></small>",
+					_("A multiplatform IRC Client"),
 					get_cpu_str(),
 					locale,
+#ifdef WIN32
+					gtk_major_version,
+					gtk_minor_version,
+					gtk_micro_version
+#else
 #ifdef USE_XFT
 					"Xft"
 #else
 					"Pango"
+#endif
 #endif
 					);
 	gtk_label_set_markup (GTK_LABEL (label), buf);
@@ -135,7 +147,7 @@ menu_about (GtkWidget * wid, gpointer sess)
 	hbox = gtk_hbox_new (0, 2);
 	gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
-	wid = gtk_button_new_from_stock (GTK_STOCK_OK);
+	wid = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (wid), GTK_CAN_DEFAULT);
 	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (about)->action_area), wid, 0, 0, 0);
 	gtk_widget_grab_default (wid);
