@@ -31,6 +31,7 @@
 #include <gtk/gtkbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtktogglebutton.h>
+#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkversion.h>
 
 #include "../common/xchat.h"
@@ -48,7 +49,6 @@
 #include "notifygui.h"
 #include "textgui.h"
 #include "fkeys.h"
-#include "tabs.h"
 #include "urlgrab.h"
 
 #ifdef USE_XLIB
@@ -75,21 +75,12 @@ redraw_trans_xtexts (void)
 	while (list)
 	{
 		sess = list->data;
-#ifdef USE_ZVT
-		if (sess->type == SESS_SHELL)
+		if (GTK_XTEXT (sess->gui->xtext)->transparent)
 		{
-			menu_newshell_set_palette (sess);
-			gtk_widget_queue_draw (sess->gui->xtext);
-		} else
-#endif
-		{
-			if (GTK_XTEXT (sess->gui->xtext)->transparent)
-			{
-				if (!sess->gui->is_tab || !done_main)
-					gtk_xtext_refresh (GTK_XTEXT (sess->gui->xtext), 1);
-				if (sess->gui->is_tab)
-					done_main = TRUE;
-			}
+			if (!sess->gui->is_tab || !done_main)
+				gtk_xtext_refresh (GTK_XTEXT (sess->gui->xtext), 1);
+			if (sess->gui->is_tab)
+				done_main = TRUE;
 		}
 		list = list->next;
 	}
@@ -98,7 +89,7 @@ redraw_trans_xtexts (void)
 static GdkFilterReturn
 root_event_cb (GdkXEvent *xev, GdkEventProperty *event, gpointer data)
 {
-	Atom at = None;
+	static Atom at = None;
 	XEvent *xevent = (XEvent *)xev;
 
 	if (xevent->type == PropertyNotify)
@@ -115,10 +106,30 @@ root_event_cb (GdkXEvent *xev, GdkEventProperty *event, gpointer data)
 
 #endif
 
+/* === command-line parameter parsing : requires glib 2.6 === */
+
+static char *arg_cfgdir = NULL;
+static gint arg_show_autoload = 0;
+static gint arg_show_config = 0;
+static gint arg_show_version = 0;
+
+static const GOptionEntry gopt_entries[] = 
+{
+ {"no-auto",	'a', 0, G_OPTION_ARG_NONE,		&arg_dont_autoconnect, N_("Don't auto connect to servers"), NULL},
+ {"cfgdir",		'd', 0, G_OPTION_ARG_STRING,	&arg_cfgdir, N_("Use a different config directory"), "PATH"},
+ {"no-plugins",'n', 0, G_OPTION_ARG_NONE,		&arg_skip_plugins, N_("Don't auto load any plugins"), NULL},
+ {"plugindir",	'p', 0, G_OPTION_ARG_NONE,		&arg_show_autoload, N_("Show plugin auto-load directory"), NULL},
+ {"configdir",	'u', 0, G_OPTION_ARG_NONE,		&arg_show_config, N_("Show user config directory"), NULL},
+ {"url",			 0,  0, G_OPTION_ARG_STRING,	&arg_url, N_("Open an irc://server:port/channel URL"), "URL"},
+ {"version",	'v', 0, G_OPTION_ARG_NONE,		&arg_show_version, N_("Show version information"), NULL},
+ {NULL}
+};
+
 int
 fe_args (int argc, char *argv[])
 {
-	int offset = 0;
+	GError *error = NULL;
+	GOptionContext *context;
 
 #ifdef ENABLE_NLS
 	bindtextdomain (PACKAGE, LOCALEDIR);
@@ -126,71 +137,47 @@ fe_args (int argc, char *argv[])
 	textdomain (PACKAGE);
 #endif
 
-	if (argc > 1)
+	context = g_option_context_new (NULL);
+	g_option_context_add_main_entries (context, gopt_entries, PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_parse (context, &argc, &argv, &error);
+
+	if (error)
+	{
+		if (error->message)
+			printf ("%s\n", error->message);
+		return 1;
+	}
+
+	g_option_context_free (context);
+
+	if (arg_show_version)
+	{
+		printf (PACKAGE" "VERSION"\n");
+		return 0;
+	}
+
+	if (arg_show_autoload)
 	{
-		if (!strcasecmp (argv[1], "-v") || !strcasecmp (argv[1], "--version"))
-		{
-			printf (PACKAGE" "VERSION"\n");
-			return 0;
-		}
-		if (!strcasecmp (argv[1], "-p"))
-		{
 #ifdef WIN32
-			/* see the chdir() below */
-			char *sl, *exe = strdup (argv[0]);
-			sl = strrchr (exe, '\\');
-			if (sl)
-			{
-				*sl = 0;
-				printf ("%s\\plugins\n", exe);
-			}
+		/* see the chdir() below */
+		char *sl, *exe = strdup (argv[0]);
+		sl = strrchr (exe, '\\');
+		if (sl)
+		{
+			*sl = 0;
+			printf ("%s\\plugins\n", exe);
+		}
 #else
-			printf ("%s\n", XCHATLIBDIR"/plugins");
+		printf ("%s\n", XCHATLIBDIR"/plugins");
 #endif
-			return 0;
-		}
-		if (!strcasecmp (argv[1], "-u"))
-		{
-			printf ("%s\n", get_xdir_fs ());
-			return 0;
-		}
-		if (!strcasecmp (argv[1], "-h") || !strcasecmp (argv[1], "--help"))
-		{
-			printf (PACKAGE" "VERSION"\n"
-					"Usage: %s [OPTIONS]... [URL]\n\n", argv[0]);
-			printf ("%s:\n"
-					"  -a,  --no-auto            %s\n"
-					"  -d,  --cfgdir %-11s %s\n"
-					"  -n,  --no-plugins         %s\n"
-					"  -p                        %s\n"
-					"  -u                        %s\n"
-					"  -v,  --version            %s\n\n"
-					"URL:\n"
-					"  irc://server:port/channel\n\n",
-						_("Options"),
-						_("don't auto connect"),
-						_("DIRECTORY"),
-						_("use a different config dir"),
-						_("don't auto load any plugins"),
-						_("show plugin auto-load dir"),
-						_("show user config dir"),
-						_("show version information")
-					);
-			return 0;
-		}
-		if (!strcasecmp (argv[1], "-a") || !strcasecmp (argv[1], "--no-auto"))
-		{
-			auto_connect = 0;
-			offset++;
-		}
-		if (argc > 1 + offset)
-		{
-			if (!strcasecmp (argv[1+offset], "-n") || !strcasecmp (argv[1+offset], "--no-plugins"))
-			{
-				skip_plugins = 1;
-				offset++;
-			}
-		}
+		return 0;
+	}
+
+	if (arg_show_config)
+	{
+		printf ("%s\n", get_xdir_fs ());
+		return 0;
 	}
 
 #ifdef WIN32
@@ -212,30 +199,23 @@ fe_args (int argc, char *argv[])
 	}
 #endif
 
-	if (argc > 2 + offset)
+	if (arg_cfgdir)	/* we want filesystem encoding */
 	{
-		if (!strcasecmp (argv[1+offset], "-d") || !strcasecmp (argv[1+offset], "--cfgdir"))
-		{
-			xdir_fs = strdup (argv[2+offset]);
-			if (xdir_fs[strlen (xdir_fs) - 1] == '/')
-				xdir_fs[strlen (xdir_fs) - 1] = 0;
-			offset += 2;
-		}
+		xdir_fs = strdup (arg_cfgdir);
+		if (xdir_fs[strlen (xdir_fs) - 1] == '/')
+			xdir_fs[strlen (xdir_fs) - 1] = 0;
+		g_free (arg_cfgdir);
 	}
 
-	if (argc > (offset + 1))
-		connect_url = strdup (argv[offset + 1]);
-
 	gtk_init (&argc, &argv);
 
 #ifdef USE_XLIB
-/*	XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), PropertyChangeMask);*/
 	gdk_window_set_events (gdk_get_default_root_window (), GDK_PROPERTY_CHANGE_MASK);
 	gdk_window_add_filter (gdk_get_default_root_window (),
 								  (GdkFilterFunc)root_event_cb, NULL);
 #endif
 
-	return 1;
+	return -1;
 }
 
 const char cursor_color_rc[] =
@@ -246,13 +226,11 @@ const char cursor_color_rc[] =
 	"widget \"*.xchat-inputbox\" style : application \"xc-ib-st\"";
 
 GtkStyle *
-create_input_style (void)
+create_input_style (GtkStyle *style)
 {
-	GtkStyle *style;
 	char buf[256];
 	static int done_rc = FALSE;
 
-	style = gtk_style_new ();
 	pango_font_description_free (style->font_desc);
 	style->font_desc = pango_font_description_from_string (prefs.font_normal);
 
@@ -260,7 +238,7 @@ create_input_style (void)
 	if (pango_font_description_get_size (style->font_desc) == 0)
 	{
 		snprintf (buf, sizeof (buf), _("Failed to open font:\n\n%s"), prefs.font_normal);
-		gtkutil_simpledialog (buf);
+		fe_message (buf, FE_MSG_ERROR);
 		pango_font_description_free (style->font_desc);
 		style->font_desc = pango_font_description_from_string ("sans 11");
 	}
@@ -288,7 +266,7 @@ fe_init (void)
 	pixmaps_init ();
 
 	channelwin_pix = pixmap_load_from_file (prefs.background);
-	input_style = create_input_style ();
+	input_style = create_input_style (gtk_style_new ());
 }
 
 void
@@ -383,25 +361,27 @@ fe_new_server (struct server *serv)
 	memset (serv->gui, 0, sizeof (struct server_gui));
 }
 
-static void
-null_this_var (GtkWidget * unused, GtkWidget ** dialog)
-{
-	*dialog = 0;
-}
-
 void
-fe_message (char *msg, int wait)
+fe_message (char *msg, int flags)
 {
 	GtkWidget *dialog;
+	int type = GTK_MESSAGE_WARNING;
 
-	dialog = gtkutil_simpledialog (msg);
-	if (wait)
-	{
-		g_signal_connect (G_OBJECT (dialog), "destroy",
-								G_CALLBACK (null_this_var), &dialog);
-		while (dialog)
-			gtk_main_iteration ();
-	}
+	if (flags & FE_MSG_ERROR)
+		type = GTK_MESSAGE_ERROR;
+	if (flags & FE_MSG_INFO)
+		type = GTK_MESSAGE_INFO;
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (parent_window), 0, type,
+												GTK_BUTTONS_OK, "%s", msg);
+	g_signal_connect (G_OBJECT (dialog), "response",
+							G_CALLBACK (gtk_widget_destroy), 0);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+	gtk_widget_show (dialog);
+
+	if (flags & FE_MSG_WAIT)
+		gtk_dialog_run (GTK_DIALOG (dialog));
 }
 
 void
@@ -544,7 +524,7 @@ void
 fe_close_window (struct session *sess)
 {
 	if (sess->gui->is_tab)
-		gtk_widget_destroy (sess->res->tab);
+		mg_tab_close (sess);
 	else
 		gtk_widget_destroy (sess->gui->window);
 }
@@ -640,7 +620,8 @@ fe_set_lag (server *serv, int lag)
 	GSList *list = sess_list;
 	session *sess;
 	gdouble per;
-	char tip[64];
+	char lagtext[64];
+	char lagtip[64];
 	unsigned long nowtim;
 
 	if (lag == -1)
@@ -655,7 +636,9 @@ fe_set_lag (server *serv, int lag)
 	if (per > 1.0)
 		per = 1.0;
 
-	snprintf (tip, sizeof (tip) - 1, "%s%d.%ds",
+	snprintf (lagtext, sizeof (lagtext) - 1, "%s%d.%ds",
+				 serv->lag_sent ? "+" : "", lag / 10, lag % 10);
+	snprintf (lagtip, sizeof (lagtip) - 1, "Lag: %s%d.%d seconds",
 				 serv->lag_sent ? "+" : "", lag / 10, lag % 10);
 
 	while (list)
@@ -663,18 +646,25 @@ fe_set_lag (server *serv, int lag)
 		sess = list->data;
 		if (sess->server == serv)
 		{
+			if (sess->res->lag_tip)
+				free (sess->res->lag_tip);
+			sess->res->lag_tip = strdup (lagtip);
+
 			if (!sess->gui->is_tab || current_tab == sess)
 			{
 				if (sess->gui->lagometer)
+				{
 					gtk_progress_bar_set_fraction ((GtkProgressBar *) sess->gui->lagometer, per);
+					add_tip (sess->gui->lagometer->parent, lagtip);
+				}
 				if (sess->gui->laginfo)
-					gtk_label_set_text ((GtkLabel *) sess->gui->laginfo, tip);
+					gtk_label_set_text ((GtkLabel *) sess->gui->laginfo, lagtext);
 			} else
 			{
 				sess->res->lag_value = per;
 				if (sess->res->lag_text)
 					free (sess->res->lag_text);
-				sess->res->lag_text = strdup (tip);
+				sess->res->lag_text = strdup (lagtext);
 			}
 		}
 		list = list->next;
@@ -688,6 +678,7 @@ fe_set_throttle (server *serv)
 	struct session *sess;
 	float per;
 	char tbuf[64];
+	char tip[64];
 
 	per = (float) serv->sendq_len / 1024.0;
 	if (per > 1.0)
@@ -699,11 +690,19 @@ fe_set_throttle (server *serv)
 		if (sess->server == serv)
 		{
 			snprintf (tbuf, sizeof (tbuf) - 1, _("%d bytes"), serv->sendq_len);
+			snprintf (tip, sizeof (tip) - 1, _("Network send queue: %d bytes"), serv->sendq_len);
+
+			if (sess->res->queue_tip)
+				free (sess->res->queue_tip);
+			sess->res->queue_tip = strdup (tip);
 
 			if (!sess->gui->is_tab || current_tab == sess)
 			{
 				if (sess->gui->throttlemeter)
+				{
 					gtk_progress_bar_set_fraction ((GtkProgressBar *) sess->gui->throttlemeter, per);
+					add_tip (sess->gui->throttlemeter->parent, tip);
+				}
 				if (sess->gui->throttleinfo)
 					gtk_label_set_text ((GtkLabel *) sess->gui->throttleinfo, tbuf);
 			} else
@@ -730,7 +729,7 @@ fe_ctrl_gui (session *sess, int action, int arg)
 		gtk_window_present (GTK_WINDOW (sess->gui->window));
 		break;
 	case 2:
-		mg_bring_tofront (sess->res->tab); break;
+		mg_bring_tofront_sess (sess); break;
 	case 3:
 		fe_set_tab_color (sess, -1, TRUE); break; /* flash */
 	case 4:
@@ -740,6 +739,8 @@ fe_ctrl_gui (session *sess, int action, int arg)
 	case 6:
 		menu_bar_toggle ();	/* toggle menubar on/off */
 		break;
+	case 7:
+		mg_detach (sess, arg);	/* arg: 0=toggle 1=detach 2=attach */
 	}
 }
 
@@ -771,6 +772,21 @@ fe_gui_info (session *sess, int info_type)
 	return -1;
 }
 
+void *
+fe_gui_info_ptr (session *sess, int info_type)
+{
+	switch (info_type)
+	{
+	case 0:	/* native window pointer (for plugins) */
+#ifdef WIN32
+		return GDK_WINDOW_HWND (sess->gui->window->window);
+#else
+		return sess->gui->window;
+#endif
+	}
+	return NULL;
+}
+
 char *
 fe_get_inputbox_contents (session *sess)
 {
@@ -815,9 +831,9 @@ fe_set_inputbox_contents (session *sess, char *text)
 		gtk_entry_set_text (GTK_ENTRY (sess->gui->input_box), text);
 	} else
 	{
-		if (sess->res->topic_text)
-			free (sess->res->topic_text);
-		sess->res->topic_text = strdup (text);
+		if (sess->res->input_text)
+			free (sess->res->input_text);
+		sess->res->input_text = strdup (text);
 	}
 }
 
@@ -872,3 +888,50 @@ fe_open_url (const char *url)
 		g_free (loc);
 	}
 }
+
+void
+fe_server_event (server *serv, int type, int arg)
+{
+	GSList *list = sess_list;
+	session *sess;
+
+	while (list)
+	{
+		sess = list->data;
+		if (sess->server == serv && (current_tab == sess || !sess->gui->is_tab))
+		{
+			session_gui *gui = sess->gui;
+
+			switch (type)
+			{
+			case FE_SE_CONNECTING:	/* connecting in progress */
+			case FE_SE_RECONDELAY:	/* reconnect delay begun */
+				/* enable Disconnect item */
+				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT], 1);
+				break;
+
+			case FE_SE_CONNECT:
+				/* enable Disconnect and Away menu items */
+				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_AWAY], 1);
+				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT], 1);
+				break;
+
+			case FE_SE_LOGGEDIN:	/* end of MOTD */
+				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], 1);
+				/* if number of auto-join channels is zero, open joind */
+				if (arg == 0)
+					joind (0, serv);
+				break;
+
+			case FE_SE_DISCONNECT:
+				/* disable Disconnect and Away menu items */
+				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_AWAY], 0);
+				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT], 0);
+				gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], 0);
+				/* close the join-dialog, if one exists */
+				joind (1, serv);
+			}
+		}
+		list = list->next;
+	}
+}
