@@ -43,6 +43,7 @@
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
 #include "../common/cfgfiles.h"
+#include "../common/fe.h"
 #include "../common/userlist.h"
 #include "../common/outbound.h"
 #include "../common/util.h"
@@ -191,8 +192,8 @@ key_init ()
 	{
 		key_load_defaults ();
 		if (key_load_kbs (NULL) == 1)
-			gtkutil_simpledialog
-				(_("There was an error loading key bindings configuration"));
+			fe_message (_("There was an error loading key"
+							" bindings configuration"), FE_MSG_ERROR);
 	}
 }
 
@@ -411,11 +412,9 @@ key_load_defaults ()
 		"CS\nPrior\nMove tab family left\nD1!\nD2!\n\n"\
 		"CS\nNext\nMove tab family right\nD1!\nD2!\n\n"\
 		"None\nF9\nRun Command\nD1:/GUI MENU TOGGLE\nD2!\n\n"
-	char buf[512];
 	int fd;
 
-	snprintf (buf, 512, "%s/keybindings.conf", get_xdir_fs ());
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+	fd = xchat_open_file ("keybindings.conf", O_CREAT | O_TRUNC | O_WRONLY, 0x180, XOF_DOMODE);
 	if (fd < 0)
 		/* ???!!! */
 		return;
@@ -697,7 +696,7 @@ key_dialog_show ()
 	}
 
 	key_dialog =
-			  mg_create_generic_tab ("editkeys", _("X-Chat: Keyboard Shortcuts"),
+			  mg_create_generic_tab ("editkeys", _("XChat: Keyboard Shortcuts"),
 							TRUE, FALSE, key_dialog_close, NULL, 560, 330, &vbox, 0);
 
 	hbox = gtk_hbox_new (0, 2);
@@ -800,7 +799,7 @@ key_dialog_show ()
 	gtk_xtext_set_tint (GTK_XTEXT (wid), prefs.tint_red, prefs.tint_green, prefs.tint_blue);
 	gtk_xtext_set_background (GTK_XTEXT (wid),
 									  channelwin_pix,
-									  prefs.transparent, prefs.tint);
+									  prefs.transparent);
 	gtk_widget_set_usize (wid, 0, 75);
 	gtk_box_pack_start (GTK_BOX (hbox2), wid, 1, 1, 1);
 	gtk_xtext_set_font (GTK_XTEXT (wid), prefs.font_normal);
@@ -824,15 +823,14 @@ key_save_kbs (char *fn)
 	struct key_binding *kb;
 
 	if (!fn)
-		snprintf (buf, 510, "%s/keybindings.conf", get_xdir_fs ());
+		fd = xchat_open_file ("keybindings.conf", O_CREAT | O_TRUNC | O_WRONLY,
+									 0x180, XOF_DOMODE);
 	else
-	{
-		safe_strcpy (buf, fn, sizeof (buf));
-	}
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+		fd = xchat_open_file (fn, O_CREAT | O_TRUNC | O_WRONLY,
+									 0x180, XOF_DOMODE | XOF_FULLPATH);
 	if (fd < 0)
 	{
-		gtkutil_simpledialog (_("Error opening keys config file\n"));
+		fe_message (_("Error opening keys config file\n"), FE_MSG_ERROR);
 		return;
 	}
 	write (fd, buf,
@@ -952,14 +950,10 @@ key_load_kbs (char *filename)
 	struct key_binding *kb = NULL, *last = NULL;
 	int fd, len, pnt = 0, state = 0, n;
 
-	buf = malloc (1000);
 	if (filename == NULL)
-		snprintf (buf, 1000, "%s/keybindings.conf", get_xdir_fs ());
+		fd = xchat_open_file ("keybindings.conf", O_RDONLY, 0, 0);
 	else
-		strcpy (buf, filename);
-
-	fd = open (buf, O_RDONLY | OFLAGS);
-	free (buf);
+		fd = xchat_open_file (filename, O_RDONLY, 0, XOF_FULLPATH);
 	if (fd < 0)
 		return 1;
 	if (fstat (fd, &st) != 0)
@@ -1005,7 +999,7 @@ key_load_kbs (char *filename)
 				snprintf (ibuf, 1024,
 							 _("Unknown keyname %s in key bindings config file\nLoad aborted, please fix %s/keybindings.conf\n"),
 							 buf, get_xdir_utf8 ());
-				gtkutil_simpledialog (ibuf);
+				fe_message (ibuf, FE_MSG_ERROR);
 				free (ibuf);
 				return 2;
 			}
@@ -1043,7 +1037,7 @@ key_load_kbs (char *filename)
 				snprintf (ibuf, 1024,
 							 _("Unknown action %s in key bindings config file\nLoad aborted, Please fix %s/keybindings\n"),
 							 buf, get_xdir_utf8 ());
-				gtkutil_simpledialog (ibuf);
+				fe_message (ibuf, FE_MSG_ERROR);
 				free (ibuf);
 				return 3;
 			}
@@ -1064,7 +1058,7 @@ key_load_kbs (char *filename)
 				snprintf (ibuf, 1024,
 							 _("Expecting Data line (beginning Dx{:|!}) but got:\n%s\n\nLoad aborted, Please fix %s/keybindings\n"),
 							 buf, get_xdir_utf8 ());
-				gtkutil_simpledialog (ibuf);
+				fe_message (ibuf, FE_MSG_ERROR);
 				free (ibuf);
 				return 4;
 			}
@@ -1134,7 +1128,7 @@ key_load_kbs (char *filename)
 						_("Key bindings config file is corrupt, load aborted\n"
 								 "Please fix %s/keybindings.conf\n"),
 						 get_xdir_utf8 ());
-	gtkutil_simpledialog (ibuf);
+	fe_message (ibuf, FE_MSG_ERROR);
 	free (ibuf);
 	return 5;
 }
@@ -1393,6 +1387,19 @@ key_action_tab_clean(void)
 /* Used in the followig completers */
 #define COMP_BUF 2048
 
+/* For use in sorting the user list for completion */
+static int
+talked_recent_cmp (struct User *a, struct User *b)
+{
+	if (a->lasttalk < b->lasttalk)
+		return -1;
+
+	if (a->lasttalk > b->lasttalk)
+		return 1;
+
+	return 0;
+}
+
 static int
 key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 							struct session *sess)
@@ -1483,6 +1490,8 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 		{
 			gcomp = g_completion_new((GCompletionFunc)gcomp_nick_func);
 			tmp_list = userlist_double_list(sess); /* create a temp list so we can free the memory */
+			if (prefs.completion_sort == 1)	/* sort in last-talk order? */
+				tmp_list = g_list_sort (tmp_list, (void *)talked_recent_cmp);
 		}
 		else
 		{
@@ -1668,7 +1677,7 @@ static int
 key_action_move_tab_left (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								  char *d2, struct session *sess)
 {
-	mg_move_tab (sess->res->tab, +1);
+	mg_move_tab (sess, +1);
 	return 2;						  /* don't allow default action */
 }
 
@@ -1676,7 +1685,7 @@ static int
 key_action_move_tab_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	mg_move_tab (sess->res->tab, -1);
+	mg_move_tab (sess, -1);
 	return 2;						  /* -''- */
 }
 
@@ -1684,7 +1693,7 @@ static int
 key_action_move_tab_family_left (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								  char *d2, struct session *sess)
 {
-	mg_move_tab_family (sess->res->tab, +1);
+	mg_move_tab_family (sess, +1);
 	return 2;						  /* don't allow default action */
 }
 
@@ -1692,7 +1701,7 @@ static int
 key_action_move_tab_family_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	mg_move_tab_family (sess->res->tab, -1);
+	mg_move_tab_family (sess, -1);
 	return 2;						  /* -''- */
 }
 
