@@ -5,10 +5,10 @@ typedef void (*filereqcallback) (void *, char *file);
 
 #define FRF_WRITE 1
 #define FRF_MULTIPLE 2
+#define FRF_ADDFOLDER 4
 
 void gtkutil_file_req (char *title, void *callback, void *userdata, char *filter, int flags);
 void gtkutil_destroy (GtkWidget * igad, GtkWidget * dgad);
-GtkWidget *gtkutil_simpledialog (char *msg);
 GtkWidget *gtkutil_button (GtkWidget *box, char *stock, char *tip, void *callback,
 				 void *userdata, char *labeltext);
 void gtkutil_label_new (char *text, GtkWidget * box);
@@ -20,6 +20,8 @@ GtkWidget *gtkutil_clist_new (int columns, char *titles[], GtkWidget * box,
 										void *unselect_callback,
 										gpointer unselect_userdata, int selection_mode);
 int gtkutil_clist_selection (GtkWidget * clist);
+int gtkutil_clist_multiple_selection (GtkWidget * clist,
+													int ** rows, const int max_rows);
 void add_tip (GtkWidget * wid, char *text);
 void show_and_unfocus (GtkWidget * wid);
 void gtkutil_set_icon (GtkWidget *win);
