@@ -34,6 +34,8 @@
 
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
+#include "../common/cfgfiles.h"
+#include "../common/server.h"
 #include "gtkutil.h"
 #include "palette.h"
 #include "maingui.h"
@@ -56,7 +58,8 @@ rawlog_save (server *serv, char *file)
 	if (file)
 	{
 		if (serv->gui->rawlog_window)
-			fh = open (file, O_TRUNC | O_WRONLY | O_CREAT, 0600);
+			fh = xchat_open_file (file, O_TRUNC | O_WRONLY | O_CREAT,
+										 0600, XOF_DOMODE | XOF_FULLPATH);
 		if (fh != -1)
 		{
 			gtk_xtext_save (GTK_XTEXT (serv->gui->rawlog_textlist), fh);
@@ -91,9 +94,9 @@ open_rawlog (struct server *serv)
 		return;
 	}
 
-	snprintf (tbuf, sizeof tbuf, _("X-Chat: Rawlog (%s)"), serv->servername);
+	snprintf (tbuf, sizeof tbuf, _("XChat: Rawlog (%s)"), serv->servername);
 	serv->gui->rawlog_window =
-		mg_create_generic_tab ("rawlog", tbuf, FALSE, TRUE, close_rawlog, serv,
+		mg_create_generic_tab ("RawLog", tbuf, FALSE, TRUE, close_rawlog, serv,
 							 640, 320, &vbox, serv);
 
 	hbox = gtk_hbox_new (FALSE, 2);
@@ -104,7 +107,7 @@ open_rawlog (struct server *serv)
 	serv->gui->rawlog_textlist = gtk_xtext_new (colors, 0);
 	gtk_xtext_set_tint (GTK_XTEXT (serv->gui->rawlog_textlist), prefs.tint_red, prefs.tint_green, prefs.tint_blue);
 	gtk_xtext_set_background (GTK_XTEXT (serv->gui->rawlog_textlist),
-									  channelwin_pix, prefs.transparent, prefs.tint);
+									  channelwin_pix, prefs.transparent);
 
 	gtk_container_add (GTK_CONTAINER (hbox), serv->gui->rawlog_textlist);
 	gtk_xtext_set_font (GTK_XTEXT (serv->gui->rawlog_textlist), prefs.font_normal);
