@@ -10,8 +10,10 @@
 
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
+#include "../common/fe.h"
 #include "../common/text.h"
 #include "../common/userlist.h"
+#include "../common/util.h"
 #include "../common/xchatc.h"
 #include "fe-gtk.h"
 #include "gtkutil.h"
@@ -47,12 +49,11 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkhscale.h>
 #ifdef WIN32
-#include <windows.h>
 #include "../common/fe.h"
 #endif
 
 
-GtkStyle *create_input_style (void);
+GtkStyle *create_input_style (GtkStyle *);
 
 #define LABEL_INDENT 6
 
@@ -96,16 +97,14 @@ static const setting textbox_settings[] =
 	{ST_EFONT,  N_("Font:"), P_OFFSETNL(font_normal), 0, 0, sizeof prefs.font_normal},
 	{ST_EFILE,  N_("Background image:"), P_OFFSETNL(background), 0, 0, sizeof prefs.background},
 	{ST_NUMBER,	N_("Scrollback lines:"), P_OFFINTNL(max_lines),0,0,100000},
-	{ST_TOGGLE, N_("Transparent background"), P_OFFINTNL(transparent),0,0,0},
+	{ST_TOGGLE, N_("Colored nick names"), P_OFFINTNL(colorednicks),
+					N_("Give each person on IRC a different color"),0,0},
 	{ST_TOGGLR, N_("Indent nick names"), P_OFFINTNL(indent_nicks),
 					N_("Make nick names right-justified"),0,0},
-	{ST_TOGGLE, N_("Tint (shade) transparency"), P_OFFINTNL(tint),0,0,0},
-	{ST_TOGGLR, N_("Colored nick names"), P_OFFINTNL(colorednicks),
-					N_("Give each person on IRC a different color"),0,0},
-	{ST_TOGGLE, N_("Strip mIRC colors"), P_OFFINTNL(stripcolor),0,0,0},
+	{ST_TOGGLE, N_("Transparent background"), P_OFFINTNL(transparent),0,0,0},
 	{ST_TOGGLR, N_("Show marker line"), P_OFFINTNL(show_marker),
 					N_("Insert a red line after the last read text."),0,0},
-	{ST_HEADER, N_("Tint Settings"), 0,0,0},
+	{ST_HEADER, N_("Transparency Settings"), 0,0,0},
 	{ST_HSCALE, N_("Red:"), P_OFFINTNL(tint_red),0,0,0},
 	{ST_HSCALE, N_("Green:"), P_OFFINTNL(tint_green),0,0,0},
 	{ST_HSCALE, N_("Blue:"), P_OFFINTNL(tint_blue),0,0,0},
@@ -135,14 +134,14 @@ static const setting inputbox_settings[] =
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
-static const char *const lagmenutext[] = 
+/*static const char *const lagmenutext[] = 
 {
 	N_("Off"),
 	N_("Graph"),
 	N_("Info text"),
 	N_("Both"),
 	NULL
-};
+};*/
 
 static const char *const ulmenutext[] = 
 {
@@ -159,20 +158,19 @@ static const setting userlist_settings[] =
 	{ST_HEADER,	N_("User List"),0,0,0},
 	{ST_MENU,	N_("User list sorted by:"), P_OFFINTNL(userlist_sort), 0, ulmenutext, 0},
 	{ST_TOGGLE, N_("Show hostnames in user list"), P_OFFINTNL(showhostname_in_userlist), 0, 0, 0},
-	{ST_TOGGLE, N_("User list buttons enabled"), P_OFFINTNL(userlistbuttons), 0, 0, 0},
 	{ST_TOGGLE, N_("Use the Text box font and colors"), P_OFFINTNL(style_namelistgad),0,0,0},
 	{ST_TOGGLE, N_("Resizable user list"), P_OFFINTNL(paned_userlist),0,0,0},
 
 	{ST_HEADER,	N_("Away tracking"),0,0,0},
-	{ST_TOGGLE,	N_("Enable away tracking"), P_OFFINTNL(away_track),0,0,0},
+	{ST_TOGGLE,	N_("Track the Away status of users and mark them in a different color"), P_OFFINTNL(away_track),0,0,0},
 	{ST_NUMBER, N_("On channels smaller than:"), P_OFFINTNL(away_size_max),0,0,10000},
 
 	{ST_HEADER,	N_("Action Upon Double Click"),0,0,0},
 	{ST_ENTRY,	N_("Execute command:"), P_OFFSETNL(doubleclickuser), 0, 0, sizeof prefs.doubleclickuser},
 
-	{ST_HEADER,	N_("Extra Gadgets"),0,0,0},
+/*	{ST_HEADER,	N_("Extra Gadgets"),0,0,0},
 	{ST_MENU,	N_("Lag meter:"), P_OFFINTNL(lagometer), 0, lagmenutext, 0},
-	{ST_MENU,	N_("Throttle meter:"), P_OFFINTNL(throttlemeter), 0, lagmenutext, 0},
+	{ST_MENU,	N_("Throttle meter:"), P_OFFINTNL(throttlemeter), 0, lagmenutext, 0},*/
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
@@ -228,6 +226,7 @@ static const setting filexfer_settings[] =
 	{ST_HEADER, N_("Files and Directories"), 0, 0, 0},
 	{ST_EOPEN,	N_("Download files to:"), P_OFFSETNL(dccdir), 0, 0, sizeof prefs.dccdir},
 	{ST_EOPEN,	N_("Move completed files to:"), P_OFFSETNL(dcc_completed_dir), 0, 0, sizeof prefs.dcc_completed_dir},
+	{ST_TOGGLE, N_("Auto accept file offers"), P_OFFINTNL(autodccsend), 0, 0, 0},
 	{ST_TOGGLE, N_("Save nick name in filenames"), P_OFFINTNL(dccwithnick), 0, 0, 0},
 	{ST_TOGGLE, N_("Convert spaces to underscore before sending"), P_OFFINTNL(dcc_send_fillspaces),0,0,0},
 
@@ -307,7 +306,7 @@ static const setting logging_settings[] =
 {
 	{ST_HEADER,	N_("Logging"),0,0,0},
 	{ST_TOGGLE,	N_("Enable logging of conversations"), P_OFFINTNL(logging), 0, 0, 0},
-	{ST_ENTRY,	N_("Log filename mask:"), P_OFFSETNL(logmask), 0, 0, sizeof prefs.logmask},
+	{ST_ENTRY,	N_("Log filename:"), P_OFFSETNL(logmask), 0, 0, sizeof prefs.logmask},
 	{ST_LABEL,	N_("(%s=Server %c=Channel %n=Network).")},
 
 	{ST_HEADER,	N_("Time Stamps"),0,0,0},
@@ -540,7 +539,7 @@ setup_menu_cb (GtkWidget *item, const setting *set)
 	}
 }
 
-#if 0
+/*
 static const char *id_strings[] =
 {
 	"",
@@ -607,7 +606,7 @@ setup_create_id_menu (GtkWidget *table, char *label, int row, char *dest)
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 }
 
-#endif
+*/
 
 static void
 setup_create_menu (GtkWidget *table, int row, const setting *set)
@@ -830,7 +829,7 @@ setup_create_frame (GtkWidget **left, GtkWidget *box)
 static void
 open_data_cb (GtkWidget *button, gpointer data)
 {
-	ShellExecute (0, "open", get_xdir_fs (), NULL, NULL, SW_SHOWNORMAL);
+	fe_open_url (get_xdir_utf8 ());
 }
 #endif
 
@@ -1519,6 +1518,7 @@ static void
 setup_create_tree (GtkWidget *box, GtkWidget *book)
 {
 	GtkWidget *tree;
+	GtkWidget *frame;
 	GtkTreeStore *model;
 	GtkTreeIter iter;
 	GtkTreeIter child_iter;
@@ -1564,8 +1564,11 @@ setup_create_tree (GtkWidget *box, GtkWidget *book)
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tree),
 							    -1, _("Categories"), renderer, "text", 0, NULL);
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (tree));
-	gtk_box_pack_start (GTK_BOX (box), tree, 0, 0, 0);
-	gtk_box_reorder_child (GTK_BOX (box), tree, 0);
+
+	frame = gtk_frame_new (NULL);
+	gtk_container_add (GTK_CONTAINER (frame), tree);
+	gtk_box_pack_start (GTK_BOX (box), frame, 0, 0, 0);
+	gtk_box_reorder_child (GTK_BOX (box), frame, 0);
 
 	if (sel_iter)
 	{
@@ -1618,7 +1621,6 @@ setup_apply (struct xchatprefs *pr)
 	GSList *list;
 	int done_main = FALSE;
 	session *sess;
-	GtkStyle *old_style;
 	int new_pix = FALSE;
 	int noapply = FALSE;
 	int do_ulist = FALSE;
@@ -1640,6 +1642,10 @@ setup_apply (struct xchatprefs *pr)
 		noapply = TRUE;
 	if (DIFF (tab_sort))
 		noapply = TRUE;
+	if (DIFF (use_server_tab))
+		noapply = TRUE;
+	if (DIFF (style_namelistgad))
+		noapply = TRUE;
 
 	if (color_change || (DIFF (away_size_max)) || (DIFF (away_track)))
 		do_ulist = TRUE;
@@ -1656,8 +1662,7 @@ setup_apply (struct xchatprefs *pr)
 		channelwin_pix = pixmap_load_from_file (prefs.background);
 	}
 
-	old_style = input_style;
-	input_style = create_input_style ();
+	input_style = create_input_style (input_style);
 
 	list = sess_list;
 	while (list)
@@ -1689,11 +1694,9 @@ setup_apply (struct xchatprefs *pr)
 
 	mg_apply_setup ();
 
-	g_object_unref (old_style);
-
 	if (noapply)
-		gtkutil_simpledialog (_("Some settings were changed that require a"
-									 " restart to take full effect."));
+		fe_message (_("Some settings were changed that require a"
+						" restart to take full effect."), FE_MSG_WARN);
 }
 
 #if 0
@@ -1720,7 +1723,7 @@ setup_window_open (void)
 {
 	GtkWidget *wid, *win, *vbox, *hbox, *hbbox;
 
-	win = gtkutil_window_new (_("X-Chat: Preferences"), "prefs", 0, 0, 3);
+	win = gtkutil_window_new (_("XChat: Preferences"), "prefs", 0, 0, 3);
 
 	vbox = gtk_vbox_new (FALSE, 5);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
