@@ -201,6 +201,28 @@ userlist_selection_list (GtkWidget *widget, int *num_ret)
 	return nicks;
 }
 
+void
+fe_userlist_set_selected (struct session *sess)
+{
+	GtkListStore *store = sess->res->user_model;
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sess->gui->user_tree));
+	GtkTreeIter iter;
+	struct User *user;
+	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter))
+	{
+		do
+		{
+			gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 3, &user, -1);
+
+			if (gtk_tree_selection_iter_is_selected (selection, &iter))
+				user->selected = 1;
+			else
+				user->selected = 0;
+				
+		} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter));
+	}
+}
+
 static GtkTreeIter *
 find_row (GtkTreeView *treeview, GtkTreeModel *model, struct User *user,
 			 int *selected)
@@ -301,32 +323,28 @@ void
 fe_userlist_insert (session *sess, struct User *newuser, int row, int sel)
 {
 	GtkTreeModel *model = sess->res->user_model;
-	GdkPixbuf *pix;
+	GdkPixbuf *pix = get_user_icon (sess->server, newuser);
 	GtkTreeIter iter;
 	int do_away = TRUE;
 
-	gtk_list_store_insert (GTK_LIST_STORE (model), &iter, row);
-	pix = get_user_icon (sess->server, newuser);
-
 	if (prefs.away_size_max < 1 || !prefs.away_track)
 		do_away = FALSE;
 
-	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-							  0, pix,
-							  1, newuser->nick,
-							  2, newuser->hostname,
-							  3, newuser,
-							  4, (do_away)
-									?	(newuser->away ? &colors[COL_AWAY] : NULL)
-									:	(NULL),
-							  -1);
+	gtk_list_store_insert_with_values (GTK_LIST_STORE (model), &iter, row,
+									0, pix,
+									1, newuser->nick,
+									2, newuser->hostname,
+									3, newuser,
+									4, (do_away)
+										?	(newuser->away ? &colors[COL_AWAY] : NULL)
+										:	(NULL),
+								  -1);
 
 	/* is it me? */
 	if (newuser->me && sess->gui->nick_box)
 	{
-		sess->res->myself = newuser;
 		if (!sess->gui->is_tab || sess == current_tab)
-			mg_set_access_icon (sess->gui, pix);
+			mg_set_access_icon (sess->gui, pix, sess->server->is_away);
 	}
 
 #if 0
@@ -357,7 +375,6 @@ void
 fe_userlist_clear (session *sess)
 {
 	gtk_list_store_clear (sess->res->user_model);
-	sess->res->myself = NULL;
 }
 
 static void
