@@ -109,12 +109,6 @@ fe_new_window (struct session *sess, int focus)
 	fe_print_text (sess, buf);
 
 	fe_print_text (sess, "\n\nCompiled in Features\0032:\017 "
-#ifdef USE_PERL
-	"Perl "
-#endif
-#ifdef USE_PYTHON
-	"Python "
-#endif
 #ifdef USE_PLUGIN
 	"Plugin "
 #endif
@@ -124,9 +118,6 @@ fe_new_window (struct session *sess, int focus)
 #ifdef USE_OPENSSL
 	"OpenSSL "
 #endif
-#ifdef SOCKS
-	"Socks5 "
-#endif
 #ifdef USE_IPV6
 	"IPv6"
 #endif
@@ -406,7 +397,7 @@ fe_args (int argc, char *argv[])
 			return 0;
 		}
 	}
-	return 1;
+	return -1;
 }
 
 void
@@ -557,7 +548,7 @@ fe_new_server (struct server *serv)
 }
 
 void
-fe_message (char *msg, int wait)
+fe_message (char *msg, int flags)
 {
 	puts (msg);
 }
@@ -565,7 +556,7 @@ fe_message (char *msg, int wait)
 void
 fe_close_window (struct session *sess)
 {
-	kill_session_callback (sess);
+	session_free (sess);
 	done = TRUE;
 }
 
@@ -676,6 +667,10 @@ fe_userlist_clear (struct session *sess)
 {
 }
 void
+fe_userlist_set_selected (struct session *sess)
+{
+}
+void
 fe_dcc_add (struct DCC *dcc)
 {
 }
@@ -803,6 +798,11 @@ fe_gui_info (session *sess, int info_type)
 {
 	return -1;
 }
+void *
+fe_gui_info_ptr (session *sess, int info_type)
+{
+	return NULL;
+}
 void fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud)
 {
 }
